/*
 * Model.java
 *
 * Created on 26 July 2000, 07:58
 */

package com.hp.hpl.mesa.rdf.exp.model;

/** Model implements a set of RDF statements.
 * @author bwm
 * @version
 */
public interface Model extends Factory {

/** computes the number of statements in the model.
 *
 * <p> returns sizeof(Statements) </p>
 * @throws RDFException
 * @return the number of statements in the model
 */
    long         size()                                     throws RDFException;    
/** Computes the set of resources which are subjects of statements in this model.
 *
 *
 * @throws RDFException
 * @return an iterator over a set of resources which are subjects of statements
 *         in the model.
 */
    ResIterator  listSubjects()                             throws RDFException;
/** Computes the set of all namespaces corresponding to predicates in the model.
 * @throws RDFException
 * @return an iterator over the set of namespaces associated with predicates in 
 *         the model.
 */
    NsIterator   listNameSpaces()                           throws RDFException;
/** Computes the set of all statements in the model
 * @throws RDFException
 * @return an iterator over all statements in the model.
 */
    StmtIterator listStatements()                           throws RDFException;
/** Computes the set of all reified statements in the model
 * @throws RDFException
 * @return an iterator over the set of all reified statements in the model
*/
    StmtIterator listReifiedStatements()                    throws RDFException;
/** constructs a resource instance linked to this model
 * @throws RDFException
 * @return a resource instance
*/    
    Resource     getResource(String uri)                    throws RDFException;
/** uses the factory supplied to create a resource instance linked to this model
 * @throws RDFException
 * @return a resource instance created by the factory provided
*/    
    Resource     getResource(String uri, RFactory f)        throws RDFException;
/** constructs a property instance linked to this model
 * @throws RDFException
 * @return a resource instance created by the factory provided
*/
    Property     getProperty(String uri)                    throws RDFException;
/** constructs a property instance linked to this model
 * @throws RDFException
 * @return a property instance linked to this model
*/    
    Property     getProperty(String namespace,
                             String localName)              throws RDFException;
/** constructs a container instance linked to this model
 * @throws RDFException
 * @return a container instance
*/    
    Container    getContainer(String uri)                   throws RDFException;
/** constructs a bag instance linked to this model
 * @throws RDFException
 * @return a bag instance
*/ 
    Bag          getBag(String uri)                         throws RDFException;
/** constructs an alt instance linked to this model
 * @throws RDFException
 * @return an alt instance
*/ 
    Alt          getAlt(String uri)                         throws RDFException;
/** constructs a seq instance linked to this model
 * @throws RDFException
 * @return a seq instance
*/ 
    Seq          getSeq(String uri)                         throws RDFException;
/** add a statement to this model
 * @throws RDFException
 * @return this model
*/     
    Model        add(Statement s)                           throws RDFException;
/** add a statement to this model
 * @throws RDFException
 * @return this model
*/ 
    Model        add(Resource s, Property p, RDFNode n)     throws RDFException;
/** import all the statements in a model into this model
 * @throws RDFException
 * @return this model
*/ 
    Model        add(Model m)                               throws RDFException;
/** removes a statement from the model
 * @throws RDFException
 * @return this model
*/ 
    Model        remove(Statement s)                        throws RDFException;
    
/** lists all the subjects with a given property
 * @throws RDFException
 * @return an iterator over the set of subjects
*/     
    ResIterator  listSubjectsWithProperty(Property p)       throws RDFException;
/** lists all the subjects with a given property and property value
 * @throws RDFException
 * @return an iterator over the set of subjects
*/ 
    ResIterator  listSubjectsWithProperty(Property p, RDFNode o)
                                                            throws RDFException;
/** lists all the nodes which are the values of a given property
 * @throws RDFException
 * @return an iterator over the set of nodes
*/ 
    NodeIterator listObjectsOfProperty(Property p)          throws RDFException;
/** lists a set of statements matching a query patern
 * @throws RDFException
 * @return an iterator over the set of statements
*/ 
    StmtIterator listStatements(Resource r, Property p, RDFNode n)
                                                            throws RDFException;
/** lists all the statements matching a given property
 * @throws RDFException
 * @return an iterator over the set of statements
*/ 
    StmtIterator listStatementsMatching(Selector s)         throws RDFException;
/** constructs a new model from the statements matching a query
 * @throws RDFException
 * @return a model containing the statements matching the query
*/     
    Model        query(Resource s, Property p, RDFNode o)   throws RDFException;
/** constructs a new model of statements matching the selector s
 * @throws RDFException
 * @return a model containing the matching statements
*/ 
    Model        queryStatementsMatching(Selector s)        throws RDFException;
/** compute the union of this model and another
 * @throws RDFException
 * @return a new model containing all the statements in either model
*/     
    Model        union(Model model)                         throws RDFException;
/** compute the intersection of this model and another
 * @throws RDFException
 * @return a new model containing the statements which are in both models
*/ 
    Model        intersection(Model model)                  throws RDFException;
/** compute the difference between two models
 * @throws RDFException
 * @return a new model containing all the statements present in this model
 *         but not in model.
*/ 
    Model        difference(Model model)                    throws RDFException;
/** mark the beginning of a transaction
 * @throws RDFException
 * @return this model
*/ 
    Model        begin()                                    throws RDFException;
/** abort the current transaction
 * @throws RDFException
 * @return this model
*/ 
    Model        abort()                                    throws RDFException;
/** commit the current transaction
 * @throws RDFException
 * @return this model
*/ 
    Model        commit()                                   throws RDFException;

/**
* @return  true if this model is indepdent of others
*/
    boolean      independent();
}